/*
 * OpenI2CRADIO
 * I/O Port Handler
 * Copyright (C) 2013-06-20 K.Ohta <whatisthis.sowhat ai gmail.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2,
 *  or (at your option) any later version.
 *  This library / program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *  See the GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this library; see the file COPYING. If not, write to the
 *  Free Software Foundation, 51 Franklin Street, Fifth Floor, Boston,
 *  MA 02110-1301, USA.
 *
 *  As a special exception, if you link this(includeed from sdcc) library
 *  with other files, some of which are compiled with SDCC,
 *  to produce an executable, this library does not by itself cause
 *  the resulting executable to be covered by the GNU General Public License.
 *  This exception does not however invalidate any other reasons why
 *  the executable file might be covered by the GNU General Public License.
 */
#include "commondef.h"
#include "ioports.h"
#include <signal.h>
#include "backlight.h"

unsigned char statecount;

void set_amfmlamp(unsigned char f)
{
    if(f == 0x00) {
        _AM_STATLAMP = 0;
        _FM_STATLAMP = 1;
    } else {
        _AM_STATLAMP = 1;
        _FM_STATLAMP = 0;
    }
}


void set_examp_mute(unsigned char f)
{
    if(f == 0x00) {
        _PORT_MUTE = 1; // 20130709 Not Mute
    } else {
        _PORT_MUTE = 0;
    }
    idle_time_ms(200);
}

void set_radiopower(unsigned char f)
{
    if(f == 0x00) {
        _PORT_RADIOPOW = 0; // OFF
    } else {
        _PORT_RADIOPOW = 1; // ON
    }
}

void set_powerlamp(unsigned char f)
{
    if(f == 0x00) {
        _POW_STATLAMP = 0;
    } else {
        _POW_STATLAMP = 1;
    }
}

#if defined(pic18f23k22) || defined(pic18f24k22) || defined(pic18f25k22) || defined(pic18f26k22) || \
    defined(_18F23K22)  || defined(_18F24K22)  || defined(_18F25K22)  || defined(_18F26K22)

/*
 * For 28Pin PIC(18F2xK22), I2C lcd using.
 */

void keyin_ioinit(void)
{
    /* Initialize IOPORTS*/
    PORTA = 0x00;
    LATA = 0x00;
    ANSELA = AN_A_VAL;
    TRISA = TRIS_A_VAL;

    PORTB = 0x00;
    LATB = 0x00;
    ANSELB = AN_B_VAL;
    TRISB = TRIS_B_VAL;

    PORTC = 0x00;
    LATC = 0x00;
    ANSELC = AN_C_VAL;
    TRISC = TRIS_C_VAL_O;
 //   lcd_backlightinit();
}
#endif

#if defined(pic18f23k20) || defined(pic18f24k20) || defined(pic18f25k20) || defined(pic18f26k20) || \
    defined(_18F23K20)  || defined(_18F24K20)  || defined(_18F25K20)  || defined(_18F26K20)

/*
 * For 28Pin PIC(18F2xK20), I2C lcd using.
 */

void keyin_ioinit(void)
{
    /* Initialize IOPORTS*/
    PORTA = 0x00;
    LATA = 0x00;
    ANSEL = 0x01; // Use RA0 AS ADC, Another is not used.
    ANSELH = 0x00; //
    TRISA = TRIS_A_VAL;

    PORTB = 0x00;
    LATB = 0x00;
    TRISB = TRIS_B_VAL;

    PORTC = 0x00;
    LATC = 0x00;
    TRISC = TRIS_C_VAL_O;
//    lcd_backlightinit();
}
#endif

#if defined(pic18f43k20) || defined(pic18f44k20) || defined(pic18f45k20) || defined(pic18f46k20) || \
    defined(_18F43K20)  || defined(_18F44K20)  || defined(_18F45K20)  || defined(_18F46K20)

/*
 * For 40Pin PIC(18F4xK20), paralell or I2C lcd using.
 */
void keyin_ioinit(void)
{
    /* Initialize IOPORTS*/
    PORTA = 0x00;
    LATA = 0x00;
    ANSEL = 0x80; // Use RA7 AS ADC, Another is not used.
    ANSELH = 0x00; //
    TRISA = TRIS_A_VAL;

    PORTB = 0x00;
    LATB = 0x00;
    TRISB = TRIS_B_VAL;

    PORTC = 0x00;
    LATC = 0x00;
    TRISC = TRIS_C_VAL_O;

    /*
     * You can use PORTD,RE0-RE2 extention, when using I2C lcd.
     */
    PORTD = 0x00;
    LATD = 0x00;
    TRISD = TRIS_D_VAL;

    TRISE = TRIS_E_VAL;
    PORTE = 0b00000000; // Mute OK

    // Interrupts
    INTCONbits.INT0IE = 0;
    INTCONbits.INT0IF = 0;
    INTCON3bits.INT1IF = 0;
    INTCON3bits.INT2IF = 0;
    INTCON3bits.INT1IE = 0;
    INTCON3bits.INT2IE = 0;

//    lcd_backlightinit();
}
#else
void keyin_ioinit(void)
{
    /* Initialize IOPORTS*/
    PORTA = 0x00;
    LATA = 0x00;
//    ANSEL = 0x01; // Use RA0 AS ADC, Another is not used.
//    ANSELH = 0x00; //
    TRISA = TRIS_A_VAL;

    PORTB = 0x00;
    LATB = 0x00;
    TRISB = TRIS_B_VAL;

    PORTC = 0x00;
    LATC = 0x00;
    TRISC = TRIS_C_VAL_O;
//    lcd_backlightinit();
}
#endif

/*
 * Read IOPORTS for KEY. You should modify if you modify circuit.
 */
void readkey_io(unsigned char state)
{
    char i;
    __bitops_t portvar;
 
    switch(state) {
        case 0:
            for(i = 0; i < 2; i++){
                keyin_old[1].byte[i] = keyin_old[0].byte[i];
                keyin_old[0].byte[i] = keyin_now.byte[i];
                keyin_now.byte[i] = 0x00;
            }
            _KEY_SENDPORT |= _KEY_SENDPORTMASK; // Group1 DOWN.
            break;
        case 1:
            _KEY_SEND_GRP1 = 0; // Group1 DOWN.
            break;
        case 2:
            portvar.byte = _KEY_RECVPORT;
            if(!_KEY_POS1MASK(portvar)) {
                keyin_now.BIT0A = 1;
            }
            if(!_KEY_POS2MASK(portvar)) {
                keyin_now.BIT1A = 1;
            }
            if(!_KEY_POS3MASK(portvar)) {
                keyin_now.BIT2A = 1;
            }
            if(!_KEY_POS4MASK(portvar)) {
                keyin_now.BIT3A = 1;
            }
            _KEY_SENDPORT |= _KEY_SENDPORTMASK; // Group1 DOWN.
            break;
        case 3:
            _KEY_SEND_GRP2 = 0; // Group2 DOWN.
            break;
        case 4:
            portvar.byte = _KEY_RECVPORT;
            if(!_KEY_POS1MASK(portvar)) {
                keyin_now.BIT0B = 1;
            }
            if(!_KEY_POS2MASK(portvar)) {
                keyin_now.BIT1B = 1;
            }
            if(!_KEY_POS3MASK(portvar)) {
                keyin_now.BIT2B = 1;
            }
            if(!_KEY_POS4MASK(portvar)) {
                keyin_now.BIT3B = 1;
            }
            _KEY_SENDPORT |= _KEY_SENDPORTMASK; // Group1 DOWN.
            break;
        case 5:
            _KEY_SEND_GRP3 = 0; // Group1 DOWN.
            break;
        case 6:
            portvar.byte = _KEY_RECVPORT;
            if(!_KEY_POS1MASK(portvar)) {
                keyin_now.BIT0C = 1;
            }
            if(!_KEY_POS2MASK(portvar)) {
                keyin_now.BIT1C = 1;
            }
            if(!_KEY_POS3MASK(portvar)) {
                keyin_now.BIT2C = 1;
            }
            if(!_KEY_POS4MASK(portvar)) {
                keyin_now.BIT3C = 1;
            }
            _KEY_SENDPORT |= _KEY_SENDPORTMASK; // Group1 DOWN.
            break;
        case 7:
            _KEY_SEND_GRP4 = 0; // Group4 up
            break;
        case 8:
            portvar.byte = _KEY_RECVPORT;
            if(!_KEY_POS1MASK(portvar)) {
                keyin_now.BIT0D = 1;
            }
            if(!_KEY_POS2MASK(portvar)) {
                keyin_now.BIT1D = 1;
            }
            if(!_KEY_POS3MASK(portvar)) {
                keyin_now.BIT2D = 1;
            }
            if(!_KEY_POS4MASK(portvar)) {
                keyin_now.BIT3D = 1;
            }
            _KEY_SENDPORT |= _KEY_SENDPORTMASK; // Group1 DOWN.
            break;
        default:
            break;
    }
}
